@@ -1,7 +1,26 @@
 ## likelihood functions for birth-death & Yule model with incomplete sampling
-## written by Liam J. Revell 2017, 2018, 2019
+## written by Liam J. Revell 2017, 2018, 2019, 2020, 2021, 2022
 ## based on likelihood functions in Stadler (2012)
 
+anova.fit.bd<-function(object,...){
+	fits<-list(...)
+	nm<-c(
+		deparse(substitute(object)),
+		if(length(fits)>0) 
+			sapply(substitute(list(...))[-1],deparse)
+	)
+	fits<-if(length(fits)==0) list(object) else c(list(object),fits)
+	logL<-sapply(fits,logLik)
+	df<-sapply(fits,function(x) attr(logLik(x),"df"))
+	AIC<-sapply(fits,AIC)
+	weight<-unclass(aic.w(AIC))
+	result<-data.frame(logL,df,AIC,weight)
+	rownames(result)<-nm
+	colnames(result)<-c("log(L)","d.f.","AIC","weight")
+	print(result)
+	invisible(result)
+}
+
 lik.bd<-function(theta,t,rho=1,N=NULL){
     lam<-theta[1]
     mu<-if(length(theta)==2) theta[2] else 0
@@ -31,7 +50,7 @@ fit.bd<-function(tree,b=NULL,d=NULL,rho=1,...){
     else init.d<-0.1*qb(tree)
 	if(hasArg(iter)) iter<-list(...)$iter
 	else iter<-10
-    if(!is.binary(tree)) tree<-multi2di(tree)
+    if(!is.binary(tree)) tree<-multi2di(tree,random=FALSE)
     T<-sort(branching.times(tree),decreasing=TRUE)
 	fit<-nlminb(c(init.b,init.d),lik.bd,t=T,rho=rho,lower=rep(0,2),
 		upper=rep(Inf,2))
@@ -45,7 +64,8 @@ fit.bd<-function(tree,b=NULL,d=NULL,rho=1,...){
 	}
     obj<-list(b=fit$par[1],d=fit$par[2],rho=rho,logL=-fit$objective,
         opt=list(counts=fit$counts,convergence=fit$convergence,
-        message=fit$message),model="birth-death")
+        message=fit$message),model="birth-death",
+		lik=function(theta) -lik.bd(theta,t=T,rho=rho))
     class(obj)<-"fit.bd"
     obj
 }
@@ -55,26 +75,32 @@ fit.bd<-function(tree,b=NULL,d=NULL,rho=1,...){
 fit.yule<-function(tree,b=NULL,d=NULL,rho=1,...){
 	if(hasArg(interval)) interval<-list(...)$interval
 	else interval<-c(0,2*qb(tree)/rho)
-    if(!is.binary(tree)) tree<-multi2di(tree)
+    if(!is.binary(tree)) tree<-multi2di(tree,random=FALSE)
     T<-sort(branching.times(tree),decreasing=TRUE)
 	fit<-optimize(lik.bd,interval,t=T,rho=rho)
 	obj<-list(b=fit$minimum,d=0,rho=rho,logL=-fit$objective,
-		opt=list(convergence=0),model="Yule")
+		opt=list(convergence=0),model="Yule",
+		lik=function(theta) -lik.bd(c(theta,0),t=T,rho=rho))
     class(obj)<-"fit.bd"
     obj
 }
 
 ## S3 print method for object class 'fit.bd'
 
+SIGNIF<-function(x,dd){
+	tens<-if(abs(x)>1) ceiling(log10(abs(x))) else 0
+	signif(x,dd+tens)
+}
+
 print.fit.bd<-function(x,...){
     if(hasArg(digits)) digits<-list(...)$digits
     else digits<-4
 	cat(paste("\nFitted",x$model,"model:\n\n"))
-    cat(paste("ML(b/lambda) =",round(x$b,digits),"\n"))
+    cat(paste("ML(b/lambda) =",SIGNIF(x$b,digits),"\n"))
     if(x$model=="birth-death") 
-		cat(paste("ML(d/mu) =",round(x$d,digits),"\n"))
-    cat(paste("log(L) =",round(x$logL,digits),"\n"))
-    cat(paste("\nAssumed sampling fraction (rho) =",x$rho,"\n"))
+		cat(paste("ML(d/mu) =",SIGNIF(x$d,digits),"\n"))
+    cat(paste("log(L) =",SIGNIF(x$logL,digits),"\n"))
+    cat(paste("\nAssumed sampling fraction (rho) =",SIGNIF(x$rho,digits),"\n"))
     if(x$opt$convergence==0) cat("\nR thinks it has converged.\n\n")
     else cat("\nR thinks optimization may not have converged.\n\n")
 }
