@@ -1,5 +1,5 @@
 ## visualize discrete time Brownian simulation on a tree
-## written by Liam J. Revell 2012, 2013, 2015, 2020
+## written by Liam J. Revell 2012, 2013, 2015, 2020, 2022
 
 bmPlot<-function(tree,type="BM",anc=0,sig2=1/1000,ngen=1000,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
@@ -62,12 +62,14 @@ bmPlot<-function(tree,type="BM",anc=0,sig2=1/1000,ngen=1000,...){
 bm<-function(X,T,...){
 	if(hasArg(bty)) bty<-list(...)$bty
 	else bty<-"o"
+	if(hasArg(lwd)) lwd<-list(...)$lwd
+	else lwd<-1
 	minX<-min(sapply(X,min))
 	maxX<-max(sapply(X,max))
 	plot(X[[1]][1],T[[1]][1],ylim=c(0,max(sapply(T,max))),
 		xlim=c(minX,maxX),ylab="time",xlab="phenotype",
 		bty=bty)
-	for(i in 1:length(X)) lines(X[[i]],T[[i]])
+	for(i in 1:length(X)) lines(X[[i]],T[[i]],lwd=lwd)
 	if(hasArg(colors)) cols<-list(...)$colors
 	else cols<-"black"
 	return(cols)
@@ -77,6 +79,8 @@ bm<-function(X,T,...){
 th<-function(X,T,...){
 	if(hasArg(bty)) bty<-list(...)$bty
 	else bty<-"o"
+	if(hasArg(lwd)) lwd<-list(...)$lwd
+	else lwd<-1
 	minX<-min(sapply(X,min))
 	maxX<-max(sapply(X,max))
 	if(hasArg(thresholds)) thresholds<-list(...)$thresholds
@@ -93,11 +97,16 @@ th<-function(X,T,...){
 		if(length(X[[i]])>1)
 			for(j in 1:(length(X[[i]])-1))
 				if(threshCol(X[[i]][j],thresholds,cols)==threshCol(X[[i]][j+1],thresholds,cols))
-					lines(X[[i]][c(j,j+1)],T[[i]][c(j,j+1)],col=threshCol(X[[i]][j+1],thresholds,cols))
+					lines(X[[i]][c(j,j+1)],T[[i]][c(j,j+1)],
+						col=threshCol(X[[i]][j+1],thresholds,cols),lwd=lwd)
 				else {
 					t<-thresholds[which(sort(c(thresholds,min(X[[i]][c(j,j+1)])))==min(X[[i]][c(j,j+1)]))]
-					lines(c(X[[i]][j],t),c(T[[i]][j],T[[i]][j]+abs(diff(c(X[[i]][j],t))/diff(X[[i]][c(j,j+1)]))),col=threshCol(X[[i]][j],thresholds,cols))
-					lines(c(X[[i]][j+1],t),c(T[[i]][j+1],T[[i]][j+1]-abs(diff(c(X[[i]][j+1],t))/diff(X[[i]][c(j,j+1)]))),col=threshCol(X[[i]][j+1],thresholds,cols))
+					lines(c(X[[i]][j],t),c(T[[i]][j],
+						T[[i]][j]+abs(diff(c(X[[i]][j],t))/diff(X[[i]][c(j,j+1)]))),
+						col=threshCol(X[[i]][j],thresholds,cols),lwd=lwd)
+					lines(c(X[[i]][j+1],t),c(T[[i]][j+1],
+						T[[i]][j+1]-abs(diff(c(X[[i]][j+1],t))/diff(X[[i]][c(j,j+1)]))),
+						col=threshCol(X[[i]][j+1],thresholds,cols),lwd=lwd)
 				}
 	for(i in 1:length(thresholds)) lines(rep(thresholds[i],2),c(0,max(sapply(T,max))),lty="dashed")
 	return(setNames(cols[1:(length(thresholds)+1)],letters[1:(length(thresholds)+1)]))
