@@ -1,5 +1,5 @@
 # function plots posterior density of mapped states from stochastic mapping
-# written by Liam J. Revell 2012, 2013, 2014, 2015, 2016, 2021
+# written by Liam J. Revell 2012, 2013, 2014, 2015, 2016, 2021, 2022, 2023
 
 densityMap<-function(trees,res=100,fsize=NULL,ftype=NULL,lwd=3,check=FALSE,legend=NULL,
 	outline=FALSE,type="phylogram",direction="rightwards",plot=TRUE,...){
@@ -82,10 +82,10 @@ densityMap<-function(trees,res=100,fsize=NULL,ftype=NULL,lwd=3,check=FALSE,legen
 
 ## S3 plot method for objects of class "densityMap"
 ## also used internally by plot.contMap
-## written by Liam J. Revell 2012, 2013, 2014, 2015, 2016, 2020
+## written by Liam J. Revell 2012, 2013, 2014, 2015, 2016, 2020, 2022, 2023
 
 plot.densityMap<-function(x,...){
-	if(class(x)=="densityMap"){
+	if(inherits(x,"densityMap")){
 		tree<-x$tree
 		cols<-x$cols
 	} else stop("x should be an object of class \"densityMap\"")
@@ -119,6 +119,8 @@ plot.densityMap<-function(x,...){
 	else ylim<-NULL
 	if(hasArg(hold)) hold<-list(...)$hold
 	else hold<-TRUE
+	if(hasArg(underscore)) underscore<-list(...)$underscore
+	else underscore<-FALSE
 	if(is.null(legend)) legend<-0.5*max(H)
 	if(is.null(fsize)) fsize<-c(1,1)
 	if(length(fsize)==1) fsize<-rep(fsize,2)
@@ -158,11 +160,12 @@ plot.densityMap<-function(x,...){
 				color=par()$fg,
 				ftype=ftype[1],xlim=xlim,
 				ylim=ylim,mar=mar,direction=direction,hold=FALSE,
-				add=direction%in%c("upwards","downwards")&&legend)
+				add=direction%in%c("upwards","downwards")&&legend,
+				underscore=underscore)
 			par(col=COL)
 		}
 		plotSimmap(tree,cols,pts=FALSE,lwd=lwd[1],fsize=fsize[1],mar=mar,ftype=ftype[1],add=outline,
-			xlim=xlim,ylim=ylim,direction=direction,offset=offset,hold=FALSE)
+			xlim=xlim,ylim=ylim,direction=direction,offset=offset,hold=FALSE,underscore=underscore)
 		if(legend){
 			ff<-function(dd){
 				if(!("."%in%dd)) dig<-0
@@ -193,12 +196,14 @@ plot.densityMap<-function(x,...){
 			par(col="white")
 			invisible(capture.output(plotTree(tree,type="fan",lwd=lwd[1]+2,
 				mar=mar,fsize=fsize[1],color=par()$fg,
-				ftype=ftype[1],xlim=xlim,ylim=ylim,hold=FALSE,offset=offset)))
+				ftype=ftype[1],xlim=xlim,ylim=ylim,hold=FALSE,offset=offset,
+				underscore=underscore)))
 			par(col=COL)
 		}
 		invisible(capture.output(plotSimmap(tree,cols,lwd=lwd[1],
 			mar=mar,fsize=fsize[1],add=outline,ftype=ftype[1],
-			type="fan",xlim=xlim,ylim=ylim,hold=FALSE,offset=offset)))
+			type="fan",xlim=xlim,ylim=ylim,hold=FALSE,offset=offset,
+			underscore=underscore)))
 		if(legend){
 			ff<-function(dd){
 				if(!("."%in%dd)) dig<-0
@@ -256,13 +261,18 @@ SetMap<-function(x,...){
 }
 
 ## drop tips from an object of class 'densityMap'
-## written by Liam J. Revell 2014, 2015
+## written by Liam J. Revell 2014, 2015, 2023
 
-drop.tip.densityMap<-function(x,tip,...){
-	if(inherits(x,"densityMap")){ 
-		class(x)<-"contMap"
-		x<-drop.tip.contMap(x,tip,...)
-		class(x)<-"densityMap"
-		return(x)
-	} else cat("x should be an object of class \"densityMap\"\n")
+drop.tip.densityMap<-function(phy,tip,...){
+	if(inherits(phy,"densityMap")){ 
+		class(phy)<-"contMap"
+		phy<-drop.tip.contMap(phy,tip,...)
+		class(phy)<-"densityMap"
+		return(phy)
+	} else cat("phy should be an object of class \"densityMap\"\n")
+}
+
+keep.tip.densityMap<-function(phy,tip,...){
+	tips<-setdiff(phy$tree$tip.label,tip)
+	drop.tip.densityMap(phy,tip=tips,...)
 }
