@@ -1,5 +1,5 @@
 ## function depends on phytools (& dependencies) and maps (& dependencies)
-## written by Liam J. Revell 2013, 2017, 2019
+## written by Liam J. Revell 2013, 2017, 2019, 2022
 
 phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
@@ -32,11 +32,11 @@ phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 }
 
 ## S3 method to plot object of class "phylo.to.map"
-## written by Liam J. Revell 2013, 2014, 2016, 2019
+## written by Liam J. Revell 2013, 2014, 2016, 2019, 2020, 2022
 
 plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	type<-type[1]
-	if(class(x)=="phylo.to.map"){
+	if(inherits(x,"phylo.to.map")){
 		tree<-x$tree
 		map<-x$map
 		coords<-x$coords
@@ -89,6 +89,10 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	else lty<-"dashed"
 	if(hasArg(pts)) pts<-list(...)$pts
 	else pts<-TRUE
+	if(hasArg(col.edge)) col.edge<-list(...)$col.edge
+	else col.edge<-rep(par()$fg,nrow(x$tree$edge))
+	if(hasArg(map.bg)) map.bg<-list(...)$map.bg
+	else map.bg<-"gray95"
 	if(type=="phylogram"){
 		if(x$direction=="downwards"&&direction=="rightwards"){
 			cat("\"phylo.to.map\" direction is \"downwards\" but plot direction has been given as \"rightwards\".\n")
@@ -120,7 +124,7 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	plot.new()
 	par(mar=mar)
 	plot.window(xlim=xlim,ylim=ylim,asp=asp)
-	map(map,add=TRUE,fill=TRUE,col="gray95",mar=rep(0,4))
+	map(map,add=TRUE,fill=TRUE,col=map.bg,mar=rep(0,4))
 	if(type=="phylogram"){
 		## preliminaries
 		cw<-reorder(tree,"cladewise")
@@ -130,7 +134,7 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			# plot a white rectangle
 			dx<-abs(diff(xlim))
 			rect(xlim[1]-1.04*dx,ylim[2]-split[1]*(ylim[2]-ylim[1]),
-				xlim[2]+1.04*dx,ylim[2],col="white",border="white")
+				xlim[2]+1.04*dx,ylim[2],col=par()$bg,border=par()$bg)
 			# rescale tree so it fits in the upper half of the plot
 			# with enough space for labels
 			pdin<-par()$din[2]
@@ -163,9 +167,10 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			for(i in 1:nrow(Y)) lines(rep(x[cw$edge[i,2]],2),Y[i,],
 				lwd=lwd[1],lend=2)
 			# plot horizontal relationships
-			for(i in 1:cw$Nnode+n) 
-				lines(range(x[cw$edge[which(cw$edge[,1]==i),2]]),
-				Y[which(cw$edge[,1]==i),1],lwd=lwd[1],lend=2)
+			for(i in 1:cw$Nnode+n){
+				ii<-which(cw$edge[,1]==i)
+				if(length(ii)>1) lines(range(x[cw$edge[ii,2]]),Y[ii,1],lwd=lwd[1],lend=2)
+			}
 			# plot tip labels
 			for(i in 1:n){ 
 				if(ftype) text(x[i],Y[which(cw$edge[,2]==i),2],
@@ -181,14 +186,14 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 				font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,
 				label.offset=fsize*strwidth(" ")/(par()$usr[2]-
 				par()$usr[1])*(par()$usr[4]-par()$usr[3]),
-				x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
-				direction=direction,tip.color="black",Ntip=Ntip(cw),
+				x.lim=xlim,y.lim=ylim,
+				direction=direction,tip.color=par()$fg,Ntip=Ntip(cw),
 				Nnode=cw$Nnode,edge=cw$edge,xx=x,yy=sapply(1:(Ntip(cw)+
 				cw$Nnode),function(x,y,z) y[match(x,z)],y=Y,z=cw$edge))
 		} else {
 			dy<-abs(diff(ylim))
 			rect(xlim[1],ylim[1],xlim[1]+split[1]*(xlim[2]-
-				xlim[1]),ylim[2],col="white",border="white")
+				xlim[1]),ylim[2],col=par()$bg,border=par()$bg)
 			sh<-fsize*strwidth(paste(" ",cw$tip.label,sep=""))+
 				0.2*fsize*strwidth("W")
 			cw$edge.length<-cw$edge.length/max(nodeHeights(cw))*
@@ -211,8 +216,10 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			}
 			points(coords,pch=pch,cex=cex.points[2],bg=colors[,2])
 			for(i in 1:nrow(X)) lines(X[i,],rep(y[cw$edge[i,2]],2),lwd=lwd[1],lend=2)
-			for(i in 1:cw$Nnode+n) lines(X[which(cw$edge[,1]==i),1],
-				range(y[cw$edge[which(cw$edge[,1]==i),2]]),lwd=lwd[1],lend=2)
+			for(i in 1:cw$Nnode+n){ 
+				ii<-which(cw$edge[,1]==i)
+				if(length(ii)>1) lines(X[ii,1],range(y[cw$edge[ii,2]]),lwd=lwd[1],lend=2)
+			}
 			for(i in 1:n){
 				if(ftype) text(X[which(cw$edge[,2]==i),2],y[i],
 					paste(" ",sub("_"," ",cw$tip.label[i]),sep=""),
@@ -223,8 +230,8 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
 				show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
 				font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=0.1,
-				x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
-				direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
+				x.lim=xlim,y.lim=ylim,
+				direction=direction,tip.color=par()$fg,Ntip=Ntip(cw),Nnode=cw$Nnode,
 				edge=cw$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
 				function(x,y,z) y[match(x,z)],y=X,z=cw$edge),yy=y)
 		}
@@ -240,7 +247,7 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 
 
 ## rotates all nodes to try and match tip an ordering 
-## written by Liam J. Revell 2013, 2015
+## written by Liam J. Revell 2013, 2015, 2020
 minRotate<-function(tree,x,...){
 	if(hasArg(print)) print<-list(...)$print
 	else print<-TRUE
@@ -248,7 +255,7 @@ minRotate<-function(tree,x,...){
 	nn<-1:tree$Nnode+Ntip(tree)
 	x<-x[tree$tip.label]
 	for(i in 1:tree$Nnode){
-		tt<-read.tree(text=write.tree(rotate(tree,nn[i])))
+		tt<-read.tree(text=write.tree(if(isSingleton(tree,nn[i])) tree else rotate(tree,nn[i])))
 		oo<-sum(abs(order(x[tree$tip.label])-1:length(tree$tip.label)))
 		pp<-sum(abs(order(x[tt$tip.label])-1:length(tt$tip.label)))
 		if(oo>pp) tree<-tt
@@ -258,6 +265,8 @@ minRotate<-function(tree,x,...){
 	return(tree)
 }
 
+isSingleton<-function(tree,node) if(sum(tree$edge[,1]==node)<=1) TRUE else FALSE
+
 print.phylo.to.map<-function(x,...){
 	cat("Object of class \"phylo.to.map\" containing:\n\n")
 	cat(paste("(1) A phylogenetic tree with",Ntip(x$tree),"tips and",x$tree$Nnode,"internal nodes.\n\n",sep=" "))
